#!/usr/bin/env python3

# updated 2005.07.21, thanks to Jacob Oscarson
# updated 2006.03.30, thanks to Mark Eichin
# updated 2016.07.19, thanks to Pahaz Blinov

import logging
import os
import subprocess
import sys
import re
import argparse

__version__ = '0.0.1'
NAME = 'py3line'
logger = logging.getLogger(NAME)
global_ctx = {
    'sh': lambda *x: subprocess.check_output(x).decode(),
    'spawn': lambda *x: 0 == subprocess.call(
        x, stderr=subprocess.STDOUT, stdout=subprocess.DEVNULL),
}


def parseargs():
    description = (
        "Pyline is a UNIX command-line tool for line-based processing "
        "in Python with regex and output transform features "
        "similar to grep, sed, and awk."
    )

    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('action',
                        metavar='action',
                        action='append', default=[],
                        help='<python_expression>')
    parser.add_argument('files', metavar='file', nargs='*',
                        help='Input file  #default: stdin')

    parser.add_argument('-a', '--action',
                        metavar='action',
                        action='append', default=[],
                        help='<python_expression>')

    parser.add_argument('-o', '--out', '--output-file',
                        dest='output', action='store', default='-',
                        help="Output file  #default: '-' for stdout")
    parser.add_argument('-i', '--in-place',
                        dest='is_inplace', action='store_true',
                        help="Output to editable file")
    parser.add_argument('--in-place-suffix',
                        dest='is_inplace_suffix', action='store', default=None,
                        help="Output to editable file and provide a backup "
                             "suffix for keeping a copy of the original file")

    parser.add_argument('-m', '--modules',
                        dest='modules',
                        action='append',
                        default=[],
                        help='for m in modules: import m  #default: []')

    parser.add_argument('-v', '--verbose',
                        dest='verbose',
                        action='store_true')
    parser.add_argument('-q', '--quiet',
                        dest='quiet',
                        action='store_true')

    parser.add_argument('--version',
                        dest='version',
                        action='store_true',
                        help='Print the version string')

    return parser.parse_args()


def x_processor(p_index, reader, action, global_ctx):
    for i, x in enumerate(reader):
        if isinstance(x, str) and x.endswith('\n'):
            x = x[:-1]

        # * if the result of any linewise expression is a boolean or None,
        #   it acts as a filter for that line (like grep)
        # * if the result of any linewise expression is a callable object,
        #   it will be passed the current value as the new value of line.
        try:
            result = eval(action, global_ctx, locals())
        except Exception as e:
            result = e

        if result is None or result is False:
            logger.debug("processor %d: skip x=%r, i=%d", p_index, x, i)
            continue
        if result is True:
            result = x

        logger.debug("processor %d: -> %r x=%r, i=%d", p_index, result, x, i)
        yield result


def _get_actions(args):
    return [compile(action.strip() or 'x', NAME, 'eval')
            for action in args.action]


def _setup_logger(args):
    if not logger.handlers:  # if no handlers, add a new one (console)
        console_handler = logging.StreamHandler()
        console_handler.setLevel(logging.DEBUG)
        console_handler.setFormatter(
            logging.Formatter('%(asctime)s | %(levelname)-8s| %(message)s')
        )
        logger.addHandler(console_handler)

    if not args.quiet:
        logger.setLevel(logging.WARN)
        if args.verbose:
            logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.ERROR)

    # capture_warnings
    logging.captureWarnings(True)
    pywarnings = logging.getLogger('py.warnings')
    pywarnings.handlers.extend(logger.handlers)


def _make_input_output_pairs(args):
    for i, o in zip(['-'], ['-']):
        i = sys.stdin if i == '-' else open(i)
        o = sys.stdout if o == '-' else open(o, mode='w')
        yield i, o


def main():
    args = parseargs()

    if args.version:
        print(__version__)
        return 0

    _setup_logger(args)
    logger.debug('arguments: %r', args)

    actions = _get_actions(args)
    pairs = _make_input_output_pairs(args)
    index = 0
    for reader, writer in pairs:
        for action in actions:
            index += 1
            reader = x_processor(index - 1, reader, action, global_ctx)
        for result in reader:
            if result is None or result is False:
                continue
            elif isinstance(result, list) or isinstance(result, tuple):
                result = ' '.join(map(str, result))
            else:
                result = str(result)
            if not result.endswith('\n'):
                result += '\n'

            writer.write(result)

    return 0


if __name__ == '__main__':
    sys.exit(main())



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

import re
from os import path
from codecs import open  # To use a consistent encoding

from setuptools import setup  # Always prefer setuptools over distutils

here = path.abspath(path.dirname(__file__))
name = 'py3line'
description = 'UNIX command-line tool for python line-based stream processing'
url = 'https://github.com/pahaz/py3line'
ppa = 'https://pypi.python.org/packages/source/s/{0}/{0}-'.format(name)

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

with open(path.join(here, name + '.py'), encoding='utf-8') as f:
    data = f.read()
    version = eval(re.search("__version__[ ]*=[ ]*([^\r\n]+)", data).group(1))

setup(
    name=name,

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=version,

    description=description,
    long_description=long_description,

    # The project's main homepage.
    url=url,
    download_url=ppa + version + '.zip',  # noqa

    # Author details
    author='Pahaz Blinov',
    author_email='pahaz.blinov@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Utilities',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    platforms=['unix', 'macos', 'windows'],

    # What does your project relate to?
    keywords='google spreadsheet api util helper',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    # packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    py_modules=["%s" % name],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
    ],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': ['check-manifest', 'docutils', 'Pygments'],
        # 'test': [
        #     'tox>=1.8.1',
        # ],
        # 'build_sphinx': [
        #     'sphinx',
        #     'sphinxcontrib-napoleon',
        # ],
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    # package_data={
    #     'tests': ['testrsa.key'],
    # },

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    # entry_points={
    #     'console_scripts': [
    #         'sshtunnel=sshtunnel:_cli_main',
    #     ]
    # },

    # Integrate tox with setuptools
    # cmdclass={'test': Tox},
)